/* athlete ADT */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "olympics.h"

static int num_athletes = 0;	/* number of athletes */

int get_num_athletes(void) { return num_athletes; }

struct athlete *create_athlete(void)
{
	struct athlete *ret;

   ret = (struct athlete *)malloc(sizeof(*ret));
   if (ret != NULL)
   { /* memory allocated - initialize */
   	memset(ret, sizeof(*ret), '\0'); /* clear memory */
      ret->id = num_athletes++;	/* number of athletes */
      ret->teams = create_link_list();	/* list of teams athlete is on */
   }

   return ret;
}

struct athlete *find_create_athlete(struct athlete *list[], char *name)
{
	char buf[BUFSIZ];	/* work aread for name */
   char *fname, *lname;	/* pointers for name */
   struct athlete *ret;

   /* parse first * last name */
   strcpy(buf, name);	/* make temp copy */
   fname=buf;
   for (lname = buf; *lname != '\0' && *lname != '|'; lname++);
   if (*lname != '\0') *lname++ = '\0'; /* terminate first name */

   /* find or create athlete */
   if ((ret = find_athlete_name(list, fname, lname)) == NULL)
   { /* not found - add athlete */
   	if ((ret = create_athlete()) != NULL)
      { /* initialize athlete */
      	list[ret->id] = ret;	/* add to list */
      	ret->fname = strdup(fname);
         ret->lname = strdup(lname);
      }
   }

   return ret;
}

struct athlete *find_athlete_name(struct athlete *list[], char *fname, char *lname)
{
	int i;

   for (i=0; i < num_athletes; i++)
   {
   	if (strcmp(list[i]->fname, fname)) continue;
      if (strcmp(list[i]->lname, lname)) continue;
      /* match found - return this */
      return list[i];
   }
   /* match not found - return NULL */
   return NULL;
}

void display_athlete(struct athlete *a)
{
	Link *link;
   struct team *t;

	printf("\t\tAthlete %d: %s %s\n", a->id, a->fname, a->lname);

   printf("\t\t\tTeams: ");
   for (link=get_head_link(a->teams); link != NULL; link = link->next)
   { /* loop thru teams */
   	t = (struct team *)get_data_from_link(link);
      if (t != NULL)
      {
      	printf(" %s", t->country);
      }
   }
   printf("\n");
}


