/* test driver for olympics CSE465 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "prodlist.h"
#include "linklist.h"

#include "olympics.h"

int nline = 0;

int main()
{
	struct event *Events[MAX_EVENTS];	/* master event list */
   int num_events=0;	/* how many actually exist */
   struct athlete *Athletes[MAX_ATHLETES];	/* master athlete list */
   int num_teams=0;			/* number teams in events */
   char buf[BUFSIZ], fname[BUFSIZ];
   FILE *fp;
   struct event *e=NULL;
   struct athlete *a=NULL;
   struct team *t=NULL;
   int running;
   int i;
   Link *link;		/* used for link list */

   strcpy(fname, "olympics.txt");	/* default file name */
   printf("Enter name of file containing events (%s): ", fname);
   gets(buf);
   if (*buf != '\0') strcpy(fname, buf);	/* new name */

   if ((fp = fopen(fname, "r")) == NULL)
   {
   	printf("Can't open input file (%s)!\n", fname);
      exit(1);
   }
   printf("Reading from file %s ...\n", fname);

   /*
    * **** Warning ****
    *
    * This code isn't robust.
    * If the file is formatted wrong, then strange things
    * can (and probably will) happen
    *
    * This code assumes that e, t, and a point to the most
    * recent event, team, and athlete
    *
    * To make it robust,
    * check for EOF after each read
    * check for NULL pointers (e.g. check e when team found)
    *
    */
   read_line(fp, buf);	/* read first line */
   running = 1;
   while (running)
   { /* process input file */
		if (*buf == '*') switch(buf[1])
      { /* command line - check what type */
      case 'e': /* event */
      case 'E':
      	if ((e = create_event()) == NULL)
         {
         	printf("Can't allocate memory for event!\n");
            exit(2);
         }
         e->id = num_events;	/* event ID number */
         Events[num_events++] = e;	/* add to list */
      	read_line(fp, buf);
         e->name = strdup(buf);	/* event name */
         read_line(fp, buf);
         e->score_type = *buf;	/* use first letter */
         read_line(fp, buf);
         e->gender = *buf;
      	read_line(fp, buf);	/* read next line */
         break;
      case 't': /* team */
      case 'T':
      	if ((t = create_team()) == NULL)
         {
         	printf("Can't allocate memory for team!\n");
            exit(3);
         }
         t->id = num_teams++;	/* team ID */
         add_ptr_prodlist(e->teams, t);	/* add to list */
         read_line(fp, buf);
         t->country = strdup(buf);
         t->event = e;	/* point back to event */
      	read_line(fp, buf);	/* read next line */
         break;
      case 'a': /* athlete */
      case 'A':
      	read_line(fp, buf); /* read first athlete */
         do
         {
      		if ((a = find_create_athlete(Athletes, buf)) == NULL)
         	{
         		printf("Can't allocate memory for athlete!\n");
            	exit(4);
         	}
				add_ptr_prodlist(t->athletes, a);
            link = create_link_entry(t);	/* link back to team */
            add_link_to_tail(a->teams, link);
				read_line(fp, buf);	/* read next line */
         } while (*buf != '*');	/* read until next command */
         break;
      case 'd': /* done */
      case 'D':
      	running = 0;
         break;
      default:
      	printf("Invalid command (%s), line %d\n", buf, nline);
      	read_line(fp, buf);	/* read next line */
      }
      else
      { /* should have been a command */
      	printf("Extraneous line in input file (%s), line %d\n", buf, nline);
      	read_line(fp, buf);	/* read next line */
      }
   }

   printf("%d events, %d teams, %d athletes\n", num_events, num_teams,
   	get_num_athletes());

   printf("File read, press RETURN to see roster: ");
   gets(buf);

   /* display all events */
   for (i=0; i < num_events; i++)
   {
   	display_event(Events[i]);
      printf("Press ENTER for next event: ");
      gets(buf);
   }

   printf("Press ENTER to terminate program: ");
   gets(buf);

	return 0;
}

/*
 * read next line from text file, ignoring comments
 */

int read_line(FILE *fp, char *buf)
{
	char *p;

   if (fgets(buf, BUFSIZ, fp) == NULL) return 0; /* EOF */
   if (*buf == '#') return read_line(fp, buf);	/* ignore comment */
   buf[BUFSIZ-1] = '\0';	/* make sure string terminated */
   for (p=buf; *p != '\0' && *p != '\n' && *p != '\r'; p++);
   *p = '\0';

   nline++;	/* increment line number (global variable) */
   return 1; /* normal return */
}


